/*
    YAM - Yet Another Mod player
    Copyright 2001, 2002 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Martin Storsj
    martin@martin.st
*/

#include "convert.h"
#include "audio.h"

int getFrameSize(void) {
	int frameSize = 1;
	if (channels==2) frameSize *= 2;
	if (bitdepth==16) frameSize *= 2;
	return frameSize;
}

void convert(unsigned char **bufferptr, int l, int r) {
	int data;

	if (channels>1) {
		/* stereo */
		if (l>0x7FFF) l = 0x7FFF;
		else if (l<-0x7FFF) l = -0x7FFF;
		if (r>0x7FFF) r = 0x7FFF;
		else if (r<-0x7FFF) r = -0x7FFF;

		if (bitdepth==16) {
			/* 16 bit */
			if (sig) {
				/* signed */
				if (littleendian) {
					/* 16 bit signed stereo little endian */
					*(*bufferptr)++ = l&0xFF;
					*(*bufferptr)++ = l>>8;
					*(*bufferptr)++ = r&0xFF;
					*(*bufferptr)++ = r>>8;
				} else {
					/* 16 bit signed stereo big endian */
					*(*bufferptr)++ = l>>8;
					*(*bufferptr)++ = l&0xFF;
					*(*bufferptr)++ = r>>8;
					*(*bufferptr)++ = r&0xFF;
				}
			} else {
				/* unsigned */
				l += 0x8000;
				r += 0x8000;
				if (littleendian) {
					/* 16 bit unsigned stereo little endian */
					*(*bufferptr)++ = l&0xFF;
					*(*bufferptr)++ = l>>8;
					*(*bufferptr)++ = r&0xFF;
					*(*bufferptr)++ = r>>8;
				} else {
					/* 16 bit unsigned stereo big endian */
					*(*bufferptr)++ = l>>8;
					*(*bufferptr)++ = l&0xFF;
					*(*bufferptr)++ = r>>8;
					*(*bufferptr)++ = r&0xFF;
				}
			}
		} else {
			/* 8 bit */
			l >>= 8;
			r >>= 8;

			if (sig) {
				/* 8 bit signed stereo */
				*(*bufferptr)++ = l&0xFF;
				*(*bufferptr)++ = r&0xFF;
			} else {
				/* 8 bit unsigned stereo */
				*(*bufferptr)++ = l+128;
				*(*bufferptr)++ = r+128;
			}
		}
	} else {
		/* mono */
		data = (l>>1)+(r>>1);
		if (data>=0x7FFF) data = 0x7FFF;
		else if (data<-0x7FFF) data = -0x7FFF;

		if (bitdepth==16) {
			/* 16 bit */
			if (sig) {
				/* signed */
				if (littleendian) {
					/* 16 bit signed mono little endian */
					*(*bufferptr)++ = data&0xFF;
					*(*bufferptr)++ = data>>8;
				} else {
					/* 16 bit signed mono big endian */
					*(*bufferptr)++ = data>>8;
					*(*bufferptr)++ = data&0xFF;
				}
			} else {
				/* unsigned */
				data += 0x8000;
				if (littleendian) {
					/* 16 bit unsigned mono little endian */
					*(*bufferptr)++ = data&0xFF;
					*(*bufferptr)++ = data>>8;
				} else {
					/* 16 bit unsigned mono big endian */
					*(*bufferptr)++ = data>>8;
					*(*bufferptr)++ = data&0xFF;
				}
			}
		} else {
			/* 8 bit */
			data >>= 8;
			if (sig) {
				/* 8 bit signed mono */
				*(*bufferptr)++ = data&0xFF;
			} else {
				/* 8 bit unsigned mono */
				*(*bufferptr)++ = data+128;
			}
		}
	}

}


