/*
    YAM - Yet Another Mod player
    Copyright 2001, 2002 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Martin Storsj
    martin@martin.st
*/

#include <stdio.h>
#include <string.h>
#include "audio.h"

#define OUTFILE "out.wav"

static FILE *out;
static int length = 0;
static int stop;
static int playing = 0;

int channels = 2;
int samplerate = 44100;
int bitdepth = 16;
int sig = 1;
int littleendian = 1;

typedef struct {
	char *id;
	int blockSize;
	char *formatName;
} riff_block;

typedef struct {
	char *signature;
	int blockSize;
	unsigned char *data;
} riff_record;

static void writeRiffBlock(FILE *out, riff_block *rb) {
	unsigned char buffer[12];
	memset(buffer,0,12);
	if (rb->id!=NULL && strlen(rb->id)>=4)
		memcpy(buffer+0,rb->id,4);
	buffer[4] = rb->blockSize&0xFF;
	buffer[5] = (rb->blockSize>>8)&0xFF;
	buffer[6] = (rb->blockSize>>16)&0xFF;
	buffer[7] = (rb->blockSize>>24)&0xFF;
	if (rb->formatName!=NULL && strlen(rb->formatName)>=4)
		memcpy(buffer+8,rb->formatName,4);
	fwrite(buffer,1,12,out);
}

static void writeRiffRecord(FILE *out, riff_record *rr) {
	unsigned char buffer[8];
	memset(buffer,0,8);
	if (rr->signature!=NULL && strlen(rr->signature)>=4)
		memcpy(buffer+0,rr->signature,4);
	buffer[4] = rr->blockSize&0xFF;
	buffer[5] = (rr->blockSize>>8)&0xFF;
	buffer[6] = (rr->blockSize>>16)&0xFF;
	buffer[7] = (rr->blockSize>>24)&0xFF;
	fwrite(buffer,1,8,out);
	if (rr->data!=NULL)
		fwrite(rr->data,1,rr->blockSize,out);
}

static void writeHeader(FILE *out, int length) {
	riff_block rb;
	riff_record fmt;
	riff_record data;
	unsigned char buffer[16];
	int bps;

	bps = 1;
	if (bitdepth==16) bps *= 2;
	if (channels==2) bps *= 2;

	rb.id = "RIFF";
	rb.blockSize = 4 + 8 + 16 + 8 + length;
	rb.formatName = "WAVE";
	writeRiffBlock(out,&rb);

	fmt.signature = "fmt ";
	fmt.blockSize = 16;
	fmt.data = buffer;
	buffer[0] = 1;					// Format
	buffer[1] = 0;					//   "
	buffer[2] = channels;				// Channels
	buffer[3] = 0;					//   "
	buffer[4] = samplerate&0x00FF;			// Samplerate
	buffer[5] = (samplerate&0xFF00)>>8;		//    "
	buffer[6] = 0;					//    "
	buffer[7] = 0;					//    "
	buffer[8] = (bps*samplerate)&0x000000FF;	// Bytes per sec
	buffer[9] = ((bps*samplerate)&0x0000FF00)>>8;	//        "
	buffer[10] = ((bps*samplerate)&0x00FF0000)>>16;	//        "
	buffer[11] = ((bps*samplerate)&0xFF000000)>>24;	//        "
	buffer[12] = bps&0x00FF;			// Bytes per frame
	buffer[13] = (bps&0xFF00)>>8;			//        "
	buffer[14] = bitdepth;				// Bits per sample
	buffer[15] = 0;					//       "
	writeRiffRecord(out,&fmt);

	data.signature = "data";
	data.blockSize = length;
	data.data = NULL;
	writeRiffRecord(out,&data);
}

int init_audio(void) {
	if ((out=fopen(OUTFILE,"wb"))==NULL) {
		perror(OUTFILE);
		return 0;
	}
	writeHeader(out,0);
	return 1;
}

void start_play(read_func rf, void *ptr) {
	unsigned char buffer[512];
	int n;
	stop = 0;
	playing = 1;
	while (!stop) {
		n = rf(ptr,buffer,sizeof(buffer));
		if (n<=0) break;
		fwrite(buffer,1,n,out);
		length += n;
	}
	playing = 0;
}

int is_playing(void) {
	return playing;
}

void break_play(void) {
	stop = 1;
}

void stop_audio(void) {
	fseek(out,0,SEEK_SET);
	writeHeader(out,length);
	fclose(out);
}

